@@ -2,11 +2,23 @@ is.installed <- function(package) {
   is.element(package, installed.packages()[,1])
 }
 
-get_java <- function() {
+get_java <- function(throws = FALSE) {
   java_home <- Sys.getenv("JAVA_HOME", unset = NA)
-  if (!is.na(java_home))
+  if (!is.na(java_home)) {
     java <- file.path(java_home, "bin", "java")
-  else
+    if (identical(.Platform$OS.type, "windows")) {
+      java <- paste0(java, ".exe")
+    }     
+    if (!file.exists(java)) {
+      if (throws) {
+        stop("Java is required to connect to Spark. ",
+             "JAVA_HOME is set but does not point to a valid version. ",
+             "Please fix JAVA_HOME or reinstall from: ",
+             java_install_url())
+      }
+      java <- ""
+    }
+  } else
     java <- Sys.which("java")
   java
 }
@@ -15,15 +27,16 @@ is_java_available <- function() {
   nzchar(get_java())
 }
 
-validate_java_version <- function(spark_home) {
-  # if somene sets SPARK_HOME, assume Java is available since some systems
+validate_java_version <- function(master, spark_home) {
+  # if someone sets SPARK_HOME and we are not in local more, assume Java
+  # is available since some systems.
   # (e.g. CDH) use versions of java not discoverable through JAVA_HOME.
-  if (!is.null(spark_home) && nchar(spark_home) > 0)
+  if (!spark_master_is_local(master) && !is.null(spark_home) && nchar(spark_home) > 0)
     return(TRUE)
 
   # find the active java executable
-  java <- get_java()
-  if (!nzchar(get_java()))
+  java <- get_java(throws = TRUE)
+  if (!nzchar(java))
     stop("Java is required to connect to Spark. Please download and install Java from ",
          java_install_url())
 
@@ -187,13 +200,19 @@ spark_sanitize_names <- function(names) {
   newNames
 }
 
-# normalize a path we are going to send to spark (pass mustWork = FALSE
-# so that e.g. hdfs:// and s3n:// paths don't produce a warning). note
+# normalizes a path that we are going to send to spark but avoids
+# normalizing remote identifiers like hdfs:// or s3n://. note
 # that this will take care of path.expand ("~") as well as converting
 # relative paths to absolute (necessary since the path will be read by
 # another process that has a different current working directory)
 spark_normalize_path <- function(path) {
-  normalizePath(path, mustWork = FALSE)
+  # don't normalize paths that are urls
+  if (grepl("[a-zA-Z]+://", path)) {
+    path
+  }
+  else {
+    normalizePath(path, mustWork = FALSE)
+  }
 }
 
 stopf <- function(fmt, ..., call. = TRUE, domain = NULL) {
@@ -269,3 +288,23 @@ sparklyr_boolean_option <- function(...) {
 sparklyr_verbose <- function(...) {
   sparklyr_boolean_option(..., "sparklyr.verbose")
 }
+
+trim_whitespace <- function(strings) {
+  gsub("^[[:space:]]*|[[:space:]]*$", "", strings)
+}
+
+
+split_separator <- function(sc) {
+  if (inherits(sc, "livy_connection"))
+    list(regexp = "\\|~\\|", plain = "|~|")
+  else
+    list(regexp = "\31", plain = "\31")
+}
+
+resolve_fn <- function(fn, ...) {
+  if (is.function(fn)) fn(...) else fn
+}
+
+is.tbl_spark <- function(x) {
+  inherits(x, "tbl_spark")
+}
